# CONFIDENTIAL. (C) 2017 OpenEye Scientific Software Inc.
# All rights reserved.
# 
# ALL SOFTWARE BELOW IS PROPRIETARY AND CONFIDENTIAL TO OPENEYE
# SCIENTIFIC SOFTWARE INC., AND IS SUBJECT TO THE FULL PROTECTION OF
# COPYRIGHT AND TRADESECRET LAW.
# Copying or modifying this software is strictly prohibited and illegal.
# Using this software without a valid license from OpenEye is strictly
# prohibited and illegal.  De-obfuscating, de-minimizing, or any other
# attempt to reverse engineer or discover the workings of this software,
# is strictly prohibited. Even if software is exposed or visible, it is
# still proprietary and confidential and the above prohibitions apply.
# This software is NOT "Sample Code". For purposes of customization or
# interoperation with other software you must use Sample Code
# specifically labeled as such.
# Please contact OpenEye at eyesopen.com/legal if you have any questions
# about this warning.


# This file was automatically generated by SWIG (http://www.swig.org).
# Version 3.0.12
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info

def swig_import_helper():
    from os.path import dirname
    import importlib.util

    spec = importlib.machinery.PathFinder().find_spec("_oechem", [dirname(__file__)])

    if spec is None:
        # OpenEye Import Statements
        spec = importlib.machinery.PathFinder().find_spec("libs",[dirname(__file__)])
    libs = importlib.util.module_from_spec(spec)

    import sys
    sys.modules[spec.name] = libs
    spec.loader.exec_module(libs)

    assert not sys.platform.startswith('win')

    _oestats = libs.OEGetModule('_oestats0965')
    return _oestats

_oestats = swig_import_helper()
del swig_import_helper
del _swig_python_version_info

try:
    _swig_property = property
except NameError:
    pass  # Python < 2.2 doesn't have 'property'.

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_setattr_nondynamic(self, class_type, name, value, static=1):
    if (name == "thisown"):
        return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name, None)
    if method:
        return method(self, value)
    if (not static):
        object.__setattr__(self, name, value)
    else:
        raise AttributeError("You cannot add attributes to %s" % self)


def _swig_setattr(self, class_type, name, value):
    return _swig_setattr_nondynamic(self, class_type, name, value, 0)


def _swig_getattr(self, class_type, name):
    if (name == "thisown"):
        return self.this.own()
    method = class_type.__swig_getmethods__.get(name, None)
    if method:
        return method(self)
    raise AttributeError("'%s' object has no attribute '%s'" % (class_type.__name__, name))


def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_method(set):
    def set_attr(self, name, value):
        if (name == "thisown"):
            return self.this.own(value)
        if hasattr(self, name) or (name == "this"):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add attributes to %s" % self)
    return set_attr


try:
    import weakref
    weakref_proxy = weakref.proxy
except __builtin__.Exception:
    weakref_proxy = lambda x: x


import openeye.oechem as oechem
OEKrigeKernelType_Linear = _oestats.OEKrigeKernelType_Linear
OEKrigeKernelType_Exponential = _oestats.OEKrigeKernelType_Exponential
OEKrigeKernelType_Gaussian = _oestats.OEKrigeKernelType_Gaussian
OEKrigeKernelType_Spherical = _oestats.OEKrigeKernelType_Spherical
OEKrigeKernelType_Cauchy = _oestats.OEKrigeKernelType_Cauchy
OEKrigeKernelType_Custom = _oestats.OEKrigeKernelType_Custom
KrigeUniqueResponseMethod_meanVal = _oestats.KrigeUniqueResponseMethod_meanVal
KrigeUniqueResponseMethod_minVal = _oestats.KrigeUniqueResponseMethod_minVal
KrigeUniqueResponseMethod_maxVal = _oestats.KrigeUniqueResponseMethod_maxVal
KrigeOptionDefaults_numLocal = _oestats.KrigeOptionDefaults_numLocal
KrigeOptionDefaults_includeUnmeasured = _oestats.KrigeOptionDefaults_includeUnmeasured
KrigeOptionDefaults_maxVariogramSize = _oestats.KrigeOptionDefaults_maxVariogramSize
KrigeOptionDefaults_predictTraining = _oestats.KrigeOptionDefaults_predictTraining
KrigeOptionDefaults_uniqueResponseMethod = _oestats.KrigeOptionDefaults_uniqueResponseMethod
class OEKrigeKernelBase(object):
    """Proxy of C++ OEStats::OEKrigeKernelBase class."""

    thisown = _swig_property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
    __repr__ = _swig_repr

    def GetName(self) -> "std::string":
        """GetName(OEKrigeKernelBase self) -> std::string"""
        return _oestats.OEKrigeKernelBase_GetName(self)


    def GetValue(self, distance: 'double') -> "double":
        """GetValue(OEKrigeKernelBase self, double distance) -> double"""
        return _oestats.OEKrigeKernelBase_GetValue(self, distance)


    def GetRange(self) -> "double":
        """GetRange(OEKrigeKernelBase self) -> double"""
        return _oestats.OEKrigeKernelBase_GetRange(self)


    def GetNugget(self) -> "double":
        """GetNugget(OEKrigeKernelBase self) -> double"""
        return _oestats.OEKrigeKernelBase_GetNugget(self)


    def GetSill(self) -> "double":
        """GetSill(OEKrigeKernelBase self) -> double"""
        return _oestats.OEKrigeKernelBase_GetSill(self)


    def SetRange(self, range: 'double') -> "OEStats::OEKrigeKernelBase &":
        """SetRange(OEKrigeKernelBase self, double range) -> OEKrigeKernelBase"""
        return _oestats.OEKrigeKernelBase_SetRange(self, range)


    def SetNugget(self, nugget: 'double') -> "OEStats::OEKrigeKernelBase &":
        """SetNugget(OEKrigeKernelBase self, double nugget) -> OEKrigeKernelBase"""
        return _oestats.OEKrigeKernelBase_SetNugget(self, nugget)


    def SetSill(self, sill: 'double') -> "OEStats::OEKrigeKernelBase &":
        """SetSill(OEKrigeKernelBase self, double sill) -> OEKrigeKernelBase"""
        return _oestats.OEKrigeKernelBase_SetSill(self, sill)


    def CreateCopy(self, *args):
        try:
            return _oestats.OEKrigeKernelBase_CreateCopy(self, *args)
        except RuntimeError:
            try:
                return self.__class__().__disown__()
            except:
                raise RuntimeError("CreateCopy must be specialized for %s" % self.__class__.__name__)


    __swig_destroy__ = _oestats.delete_OEKrigeKernelBase
    __del__ = lambda self: None

    def __init__(self):
        """__init__(OEStats::OEKrigeKernelBase self) -> OEKrigeKernelBase"""
        if self.__class__ == OEKrigeKernelBase:
            _self = None
        else:
            _self = self
        this = _oestats.new_OEKrigeKernelBase(_self, )
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    def __disown__(self):
        self.this.disown()
        _oestats.disown_OEKrigeKernelBase(self)
        return weakref_proxy(self)
OEKrigeKernelBase_swigregister = _oestats.OEKrigeKernelBase_swigregister
OEKrigeKernelBase_swigregister(OEKrigeKernelBase)

class OEKrigeKernel(OEKrigeKernelBase):
    """Proxy of C++ OEStats::OEKrigeKernel class."""

    thisown = _swig_property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
    __repr__ = _swig_repr

    def GetName(self) -> "std::string":
        """GetName(OEKrigeKernel self) -> std::string"""
        return _oestats.OEKrigeKernel_GetName(self)


    def GetValue(self, distance: 'double') -> "double":
        """GetValue(OEKrigeKernel self, double distance) -> double"""
        return _oestats.OEKrigeKernel_GetValue(self, distance)


    def GetRange(self) -> "double":
        """GetRange(OEKrigeKernel self) -> double"""
        return _oestats.OEKrigeKernel_GetRange(self)


    def GetNugget(self) -> "double":
        """GetNugget(OEKrigeKernel self) -> double"""
        return _oestats.OEKrigeKernel_GetNugget(self)


    def GetSill(self) -> "double":
        """GetSill(OEKrigeKernel self) -> double"""
        return _oestats.OEKrigeKernel_GetSill(self)


    def SetRange(self, range: 'double') -> "OEStats::OEKrigeKernelBase &":
        """SetRange(OEKrigeKernel self, double range) -> OEKrigeKernelBase"""
        return _oestats.OEKrigeKernel_SetRange(self, range)


    def SetNugget(self, nugget: 'double') -> "OEStats::OEKrigeKernelBase &":
        """SetNugget(OEKrigeKernel self, double nugget) -> OEKrigeKernelBase"""
        return _oestats.OEKrigeKernel_SetNugget(self, nugget)


    def SetSill(self, sill: 'double') -> "OEStats::OEKrigeKernelBase &":
        """SetSill(OEKrigeKernel self, double sill) -> OEKrigeKernelBase"""
        return _oestats.OEKrigeKernel_SetSill(self, sill)


    def CreateCopy(self, *args):
        try:
            return _oestats.OEKrigeKernel_CreateCopy(self, *args)
        except RuntimeError:
            try:
                return self.__class__().__disown__()
            except:
                raise RuntimeError("CreateCopy must be specialized for %s" % self.__class__.__name__)



    def __init__(self, *args):
        """
        __init__(OEStats::OEKrigeKernel self, unsigned int type) -> OEKrigeKernel
        __init__(OEStats::OEKrigeKernel self) -> OEKrigeKernel
        __init__(OEStats::OEKrigeKernel self, OEKrigeKernelBase kernel) -> OEKrigeKernel
        """
        this = _oestats.new_OEKrigeKernel(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _oestats.delete_OEKrigeKernel
    __del__ = lambda self: None
OEKrigeKernel_swigregister = _oestats.OEKrigeKernel_swigregister
OEKrigeKernel_swigregister(OEKrigeKernel)

class OEVariogram(object):
    """Proxy of C++ OEStats::OEVariogram class."""

    thisown = _swig_property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
    __repr__ = _swig_repr

    def GetNumBins(self) -> "unsigned int":
        """GetNumBins(OEVariogram self) -> unsigned int"""
        return _oestats.OEVariogram_GetNumBins(self)


    def GetDistance(self, iBin: 'unsigned int') -> "double":
        """GetDistance(OEVariogram self, unsigned int iBin) -> double"""
        return _oestats.OEVariogram_GetDistance(self, iBin)


    def GetSemivariance(self, iBin: 'unsigned int') -> "double":
        """GetSemivariance(OEVariogram self, unsigned int iBin) -> double"""
        return _oestats.OEVariogram_GetSemivariance(self, iBin)


    def __init__(self, *args):
        """
        __init__(OEStats::OEVariogram self) -> OEVariogram
        __init__(OEStats::OEVariogram self, OEStats::OEVariogramBase * impl) -> OEVariogram
        __init__(OEStats::OEVariogram self, OEVariogram rhs) -> OEVariogram
        """
        this = _oestats.new_OEVariogram(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _oestats.delete_OEVariogram
    __del__ = lambda self: None
OEVariogram_swigregister = _oestats.OEVariogram_swigregister
OEVariogram_swigregister(OEVariogram)


def OEFitKernel(kernel: 'OEKrigeKernelBase', variogram: 'OEVariogram') -> "double":
    """OEFitKernel(OEKrigeKernelBase kernel, OEVariogram variogram) -> double"""
    return _oestats.OEFitKernel(kernel, variogram)

def OEFitKernelPartial(kernel: 'OEKrigeKernelBase', variogram: 'OEVariogram', fracPoints: 'double') -> "double":
    """OEFitKernelPartial(OEKrigeKernelBase kernel, OEVariogram variogram, double fracPoints) -> double"""
    return _oestats.OEFitKernelPartial(kernel, variogram, fracPoints)

def OEGetExperimentalVariance(arg1: 'OEVariogram') -> "double":
    """OEGetExperimentalVariance(OEVariogram arg1) -> double"""
    return _oestats.OEGetExperimentalVariance(arg1)

def OEGetFitness(kernel: 'OEKrigeKernelBase', variogram: 'OEVariogram') -> "double":
    """OEGetFitness(OEKrigeKernelBase kernel, OEVariogram variogram) -> double"""
    return _oestats.OEGetFitness(kernel, variogram)
class OEGlobalKrigeOptions(object):
    """Proxy of C++ OEStats::OEGlobalKrigeOptions class."""

    thisown = _swig_property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
    __repr__ = _swig_repr

    def GetIncludeUnmeasured(self) -> "bool":
        """GetIncludeUnmeasured(OEGlobalKrigeOptions self) -> bool"""
        return _oestats.OEGlobalKrigeOptions_GetIncludeUnmeasured(self)


    def GetUniqueResponseMethod(self) -> "unsigned int":
        """GetUniqueResponseMethod(OEGlobalKrigeOptions self) -> unsigned int"""
        return _oestats.OEGlobalKrigeOptions_GetUniqueResponseMethod(self)


    def SetIncludeUnmeasured(self, *args) -> "OEStats::OEGlobalKrigeOptions &":
        """
        SetIncludeUnmeasured(OEGlobalKrigeOptions self, bool arg2) -> OEGlobalKrigeOptions
        SetIncludeUnmeasured(OEGlobalKrigeOptions self) -> OEGlobalKrigeOptions
        """
        return _oestats.OEGlobalKrigeOptions_SetIncludeUnmeasured(self, *args)


    def SetUniqueResponseMethod(self, *args) -> "OEStats::OEGlobalKrigeOptions &":
        """
        SetUniqueResponseMethod(OEGlobalKrigeOptions self, unsigned int arg2) -> OEGlobalKrigeOptions
        SetUniqueResponseMethod(OEGlobalKrigeOptions self) -> OEGlobalKrigeOptions
        """
        return _oestats.OEGlobalKrigeOptions_SetUniqueResponseMethod(self, *args)


    def __init__(self):
        """__init__(OEStats::OEGlobalKrigeOptions self) -> OEGlobalKrigeOptions"""
        this = _oestats.new_OEGlobalKrigeOptions()
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _oestats.delete_OEGlobalKrigeOptions
    __del__ = lambda self: None
OEGlobalKrigeOptions_swigregister = _oestats.OEGlobalKrigeOptions_swigregister
OEGlobalKrigeOptions_swigregister(OEGlobalKrigeOptions)

class OELocalKrigeOptions(object):
    """Proxy of C++ OEStats::OELocalKrigeOptions class."""

    thisown = _swig_property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
    __repr__ = _swig_repr

    def GetIncludeUnmeasured(self) -> "bool":
        """GetIncludeUnmeasured(OELocalKrigeOptions self) -> bool"""
        return _oestats.OELocalKrigeOptions_GetIncludeUnmeasured(self)


    def GetUniqueResponseMethod(self) -> "unsigned int":
        """GetUniqueResponseMethod(OELocalKrigeOptions self) -> unsigned int"""
        return _oestats.OELocalKrigeOptions_GetUniqueResponseMethod(self)


    def GetNumLocal(self) -> "unsigned int":
        """GetNumLocal(OELocalKrigeOptions self) -> unsigned int"""
        return _oestats.OELocalKrigeOptions_GetNumLocal(self)


    def GetMaxVariogramSize(self) -> "unsigned int":
        """GetMaxVariogramSize(OELocalKrigeOptions self) -> unsigned int"""
        return _oestats.OELocalKrigeOptions_GetMaxVariogramSize(self)


    def GetPredictTraining(self) -> "bool":
        """GetPredictTraining(OELocalKrigeOptions self) -> bool"""
        return _oestats.OELocalKrigeOptions_GetPredictTraining(self)


    def SetIncludeUnmeasured(self, *args) -> "OEStats::OELocalKrigeOptions &":
        """
        SetIncludeUnmeasured(OELocalKrigeOptions self, bool arg2) -> OELocalKrigeOptions
        SetIncludeUnmeasured(OELocalKrigeOptions self) -> OELocalKrigeOptions
        """
        return _oestats.OELocalKrigeOptions_SetIncludeUnmeasured(self, *args)


    def SetNumLocal(self, *args) -> "OEStats::OELocalKrigeOptions &":
        """
        SetNumLocal(OELocalKrigeOptions self, unsigned int arg2) -> OELocalKrigeOptions
        SetNumLocal(OELocalKrigeOptions self) -> OELocalKrigeOptions
        """
        return _oestats.OELocalKrigeOptions_SetNumLocal(self, *args)


    def SetUniqueResponseMethod(self, *args) -> "OEStats::OELocalKrigeOptions &":
        """
        SetUniqueResponseMethod(OELocalKrigeOptions self, unsigned int arg2) -> OELocalKrigeOptions
        SetUniqueResponseMethod(OELocalKrigeOptions self) -> OELocalKrigeOptions
        """
        return _oestats.OELocalKrigeOptions_SetUniqueResponseMethod(self, *args)


    def SetMaxVariogramSize(self, *args) -> "OEStats::OELocalKrigeOptions &":
        """
        SetMaxVariogramSize(OELocalKrigeOptions self, unsigned int arg2) -> OELocalKrigeOptions
        SetMaxVariogramSize(OELocalKrigeOptions self) -> OELocalKrigeOptions
        """
        return _oestats.OELocalKrigeOptions_SetMaxVariogramSize(self, *args)


    def SetPredictTraining(self, *args) -> "OEStats::OELocalKrigeOptions &":
        """
        SetPredictTraining(OELocalKrigeOptions self, bool arg2) -> OELocalKrigeOptions
        SetPredictTraining(OELocalKrigeOptions self) -> OELocalKrigeOptions
        """
        return _oestats.OELocalKrigeOptions_SetPredictTraining(self, *args)


    def __init__(self):
        """__init__(OEStats::OELocalKrigeOptions self) -> OELocalKrigeOptions"""
        this = _oestats.new_OELocalKrigeOptions()
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _oestats.delete_OELocalKrigeOptions
    __del__ = lambda self: None
OELocalKrigeOptions_swigregister = _oestats.OELocalKrigeOptions_swigregister
OELocalKrigeOptions_swigregister(OELocalKrigeOptions)

class OEDefaultKrigeDistanceOptions(object):
    """Proxy of C++ OEStats::OEDefaultKrigeDistanceOptions class."""

    thisown = _swig_property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
    __repr__ = _swig_repr

    def GetMaxPathLength(self) -> "unsigned int":
        """GetMaxPathLength(OEDefaultKrigeDistanceOptions self) -> unsigned int"""
        return _oestats.OEDefaultKrigeDistanceOptions_GetMaxPathLength(self)


    def GetMinPathLength(self) -> "unsigned int":
        """GetMinPathLength(OEDefaultKrigeDistanceOptions self) -> unsigned int"""
        return _oestats.OEDefaultKrigeDistanceOptions_GetMinPathLength(self)


    def GetForcePathsOut(self) -> "bool":
        """GetForcePathsOut(OEDefaultKrigeDistanceOptions self) -> bool"""
        return _oestats.OEDefaultKrigeDistanceOptions_GetForcePathsOut(self)


    def GetWeightByLength(self) -> "bool":
        """GetWeightByLength(OEDefaultKrigeDistanceOptions self) -> bool"""
        return _oestats.OEDefaultKrigeDistanceOptions_GetWeightByLength(self)


    def GetStereoAware(self) -> "bool":
        """GetStereoAware(OEDefaultKrigeDistanceOptions self) -> bool"""
        return _oestats.OEDefaultKrigeDistanceOptions_GetStereoAware(self)


    def SetMaxPathLength(self, arg2: 'unsigned int') -> "OEStats::OEDefaultKrigeDistanceOptions &":
        """SetMaxPathLength(OEDefaultKrigeDistanceOptions self, unsigned int arg2) -> OEDefaultKrigeDistanceOptions"""
        return _oestats.OEDefaultKrigeDistanceOptions_SetMaxPathLength(self, arg2)


    def SetMinPathLength(self, arg2: 'unsigned int') -> "OEStats::OEDefaultKrigeDistanceOptions &":
        """SetMinPathLength(OEDefaultKrigeDistanceOptions self, unsigned int arg2) -> OEDefaultKrigeDistanceOptions"""
        return _oestats.OEDefaultKrigeDistanceOptions_SetMinPathLength(self, arg2)


    def SetForcePathsOut(self, arg2: 'bool') -> "OEStats::OEDefaultKrigeDistanceOptions &":
        """SetForcePathsOut(OEDefaultKrigeDistanceOptions self, bool arg2) -> OEDefaultKrigeDistanceOptions"""
        return _oestats.OEDefaultKrigeDistanceOptions_SetForcePathsOut(self, arg2)


    def SetWeightByLength(self, arg2: 'bool') -> "OEStats::OEDefaultKrigeDistanceOptions &":
        """SetWeightByLength(OEDefaultKrigeDistanceOptions self, bool arg2) -> OEDefaultKrigeDistanceOptions"""
        return _oestats.OEDefaultKrigeDistanceOptions_SetWeightByLength(self, arg2)


    def SetStereoAware(self, arg2: 'bool') -> "OEStats::OEDefaultKrigeDistanceOptions &":
        """SetStereoAware(OEDefaultKrigeDistanceOptions self, bool arg2) -> OEDefaultKrigeDistanceOptions"""
        return _oestats.OEDefaultKrigeDistanceOptions_SetStereoAware(self, arg2)


    def GetImpl(self) -> "OEGraphSim::OEVseprOptions const *":
        """GetImpl(OEDefaultKrigeDistanceOptions self) -> OEGraphSim::OEVseprOptions const *"""
        return _oestats.OEDefaultKrigeDistanceOptions_GetImpl(self)


    def __init__(self, *args):
        """
        __init__(OEStats::OEDefaultKrigeDistanceOptions self) -> OEDefaultKrigeDistanceOptions
        __init__(OEStats::OEDefaultKrigeDistanceOptions self, OEDefaultKrigeDistanceOptions rhs) -> OEDefaultKrigeDistanceOptions
        """
        this = _oestats.new_OEDefaultKrigeDistanceOptions(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _oestats.delete_OEDefaultKrigeDistanceOptions
    __del__ = lambda self: None
OEDefaultKrigeDistanceOptions_swigregister = _oestats.OEDefaultKrigeDistanceOptions_swigregister
OEDefaultKrigeDistanceOptions_swigregister(OEDefaultKrigeDistanceOptions)

class OEMolKrigeResult(object):
    """Proxy of C++ OEStats::OEMolKrigeResult class."""

    thisown = _swig_property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
    __repr__ = _swig_repr

    def Valid(self) -> "bool":
        """Valid(OEMolKrigeResult self) -> bool"""
        return _oestats.OEMolKrigeResult_Valid(self)


    def GetResponse(self) -> "double":
        """GetResponse(OEMolKrigeResult self) -> double"""
        return _oestats.OEMolKrigeResult_GetResponse(self)


    def GetVariance(self) -> "double":
        """GetVariance(OEMolKrigeResult self) -> double"""
        return _oestats.OEMolKrigeResult_GetVariance(self)


    def GetAtomContribution(self, *args) -> "double":
        """
        GetAtomContribution(OEMolKrigeResult self, OEAtomBase atom) -> double
        GetAtomContribution(OEMolKrigeResult self, OEAtomBase atom) -> double
        GetAtomContribution(OEMolKrigeResult self, OEAtomBase atom, double baseline) -> double
        GetAtomContribution(OEMolKrigeResult self, OEAtomBase atom, double baseline) -> double
        """
        return _oestats.OEMolKrigeResult_GetAtomContribution(self, *args)


    def GetKernel(self) -> "OEStats::OEKrigeKernel":
        """GetKernel(OEMolKrigeResult self) -> OEKrigeKernel"""
        return _oestats.OEMolKrigeResult_GetKernel(self)


    def GetVariogram(self) -> "OEStats::OEVariogram":
        """GetVariogram(OEMolKrigeResult self) -> OEVariogram"""
        return _oestats.OEMolKrigeResult_GetVariogram(self)


    def GetNumElements(self) -> "unsigned int":
        """GetNumElements(OEMolKrigeResult self) -> unsigned int"""
        return _oestats.OEMolKrigeResult_GetNumElements(self)


    def GetElementDistance(self, iElement: 'unsigned int') -> "double":
        """GetElementDistance(OEMolKrigeResult self, unsigned int iElement) -> double"""
        return _oestats.OEMolKrigeResult_GetElementDistance(self, iElement)


    def GetElementWeight(self, iElement: 'unsigned int') -> "double":
        """GetElementWeight(OEMolKrigeResult self, unsigned int iElement) -> double"""
        return _oestats.OEMolKrigeResult_GetElementWeight(self, iElement)


    def GetElementResponse(self, iElement: 'unsigned int') -> "double":
        """GetElementResponse(OEMolKrigeResult self, unsigned int iElement) -> double"""
        return _oestats.OEMolKrigeResult_GetElementResponse(self, iElement)


    def GetElementIsUnmeasured(self, iElement: 'unsigned int') -> "bool":
        """GetElementIsUnmeasured(OEMolKrigeResult self, unsigned int iElement) -> bool"""
        return _oestats.OEMolKrigeResult_GetElementIsUnmeasured(self, iElement)


    def GetElement(self, iElement: 'unsigned int') -> "OEChem::OEMolBase const &":
        """GetElement(OEMolKrigeResult self, unsigned int iElement) -> OEMolBase"""
        return _oestats.OEMolKrigeResult_GetElement(self, iElement)


    def GetTrainingResponses(self, incUnmeasured: 'bool'=True) -> "OESystem::OEIterBase< double > *":
        """
        GetTrainingResponses(OEMolKrigeResult self, bool incUnmeasured=True) -> OESystem::OEIterBase< double >
        GetTrainingResponses(OEMolKrigeResult self) -> OESystem::OEIterBase< double > *
        """
        return _oestats.OEMolKrigeResult_GetTrainingResponses(self, incUnmeasured)


    def __init__(self, *args):
        """
        __init__(OEStats::OEMolKrigeResult self) -> OEMolKrigeResult
        __init__(OEStats::OEMolKrigeResult self, OEStats::OEMolKrigeResultImpl * impl) -> OEMolKrigeResult
        __init__(OEStats::OEMolKrigeResult self, OEMolKrigeResult rhs) -> OEMolKrigeResult
        """
        this = _oestats.new_OEMolKrigeResult(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _oestats.delete_OEMolKrigeResult
    __del__ = lambda self: None
OEMolKrigeResult_swigregister = _oestats.OEMolKrigeResult_swigregister
OEMolKrigeResult_swigregister(OEMolKrigeResult)


def OEProbabilityLessThan(arg1: 'OEMolKrigeResult', val: 'double') -> "double":
    """OEProbabilityLessThan(OEMolKrigeResult arg1, double val) -> double"""
    return _oestats.OEProbabilityLessThan(arg1, val)

def OEProbabilityGreaterThan(arg1: 'OEMolKrigeResult', val: 'double') -> "double":
    """OEProbabilityGreaterThan(OEMolKrigeResult arg1, double val) -> double"""
    return _oestats.OEProbabilityGreaterThan(arg1, val)

def OEProbabilityBetween(arg1: 'OEMolKrigeResult', val1: 'double', val2: 'double') -> "double":
    """OEProbabilityBetween(OEMolKrigeResult arg1, double val1, double val2) -> double"""
    return _oestats.OEProbabilityBetween(arg1, val1, val2)
class OEMolKrige(object):
    """Proxy of C++ OEStats::OEMolKrige class."""

    thisown = _swig_property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
    __repr__ = _swig_repr

    def AddTraining(self, *args) -> "bool":
        """
        AddTraining(OEMolKrige self, OEMolBase arg2, double response) -> bool
        AddTraining(OEMolKrige self, OEMCMolBase arg2, double response) -> bool
        """
        return _oestats.OEMolKrige_AddTraining(self, *args)


    def AddUnmeasuredTraining(self, *args) -> "bool":
        """
        AddUnmeasuredTraining(OEMolKrige self, OEMolBase arg2, double response) -> bool
        AddUnmeasuredTraining(OEMolKrige self, OEMCMolBase arg2, double response) -> bool
        """
        return _oestats.OEMolKrige_AddUnmeasuredTraining(self, *args)


    def Train(self, *args) -> "bool":
        """
        Train(OEMolKrige self) -> bool
        Train(OEMolKrige self, OEKrigeKernel arg2) -> bool
        """
        return _oestats.OEMolKrige_Train(self, *args)


    def TrainFixed(self, arg2: 'OEKrigeKernel') -> "bool":
        """TrainFixed(OEMolKrige self, OEKrigeKernel arg2) -> bool"""
        return _oestats.OEMolKrige_TrainFixed(self, arg2)


    def GetResult(self, *args) -> "OEStats::OEMolKrigeResult":
        """
        GetResult(OEMolKrige self, OEMolBase arg2) -> OEMolKrigeResult
        GetResult(OEMolKrige self, OEMolBase arg2) -> OEMolKrigeResult
        GetResult(OEMolKrige self, OEMCMolBase arg2) -> OEMolKrigeResult
        GetResult(OEMolKrige self, OEMCMolBase arg2) -> OEMolKrigeResult
        """
        return _oestats.OEMolKrige_GetResult(self, *args)


    def GetVariogram(self, *args) -> "OEStats::OEVariogram":
        """
        GetVariogram(OEMolKrige self) -> OEVariogram
        GetVariogram(OEMolKrige self) -> OEVariogram
        """
        return _oestats.OEMolKrige_GetVariogram(self, *args)


    def Clear(self) -> "void":
        """Clear(OEMolKrige self)"""
        return _oestats.OEMolKrige_Clear(self)


    def __init__(self, *args):
        """
        __init__(OEStats::OEMolKrige self, OEGlobalKrigeOptions arg2) -> OEMolKrige
        __init__(OEStats::OEMolKrige self) -> OEMolKrige
        __init__(OEStats::OEMolKrige self, OELocalKrigeOptions arg2) -> OEMolKrige
        __init__(OEStats::OEMolKrige self, OEDefaultKrigeDistanceOptions arg2, OEGlobalKrigeOptions arg3) -> OEMolKrige
        __init__(OEStats::OEMolKrige self, OEDefaultKrigeDistanceOptions arg2) -> OEMolKrige
        __init__(OEStats::OEMolKrige self, OEDefaultKrigeDistanceOptions arg2, OELocalKrigeOptions arg3) -> OEMolKrige
        __init__(OEStats::OEMolKrige self, OEMolPropertyList trendPropFxn, OEGlobalKrigeOptions arg3) -> OEMolKrige
        __init__(OEStats::OEMolKrige self, OEMolPropertyList trendPropFxn) -> OEMolKrige
        __init__(OEStats::OEMolKrige self, OEMolPropertyList trendPropFxn, OELocalKrigeOptions arg3) -> OEMolKrige
        __init__(OEStats::OEMolKrige self, OEDefaultKrigeDistanceOptions arg2, OEMolPropertyList trendPropFxn, OEGlobalKrigeOptions arg4) -> OEMolKrige
        __init__(OEStats::OEMolKrige self, OEDefaultKrigeDistanceOptions arg2, OEMolPropertyList trendPropFxn) -> OEMolKrige
        __init__(OEStats::OEMolKrige self, OEDefaultKrigeDistanceOptions arg2, OEMolPropertyList trendPropFxn, OELocalKrigeOptions arg4) -> OEMolKrige
        __init__(OEStats::OEMolKrige self, OEMolKrigeDistFxn distFxn, OEGlobalKrigeOptions arg3) -> OEMolKrige
        __init__(OEStats::OEMolKrige self, OEMolKrigeDistFxn distFxn) -> OEMolKrige
        __init__(OEStats::OEMolKrige self, OEMolKrigeDistFxn distFxn, OELocalKrigeOptions arg3) -> OEMolKrige
        __init__(OEStats::OEMolKrige self, OEMolKrigeDistFxn distFxn, OEMolPropertyList trendPropFxn, OEGlobalKrigeOptions arg4) -> OEMolKrige
        __init__(OEStats::OEMolKrige self, OEMolKrigeDistFxn distFxn, OEMolPropertyList trendPropFxn) -> OEMolKrige
        __init__(OEStats::OEMolKrige self, OEMolKrigeDistFxn distFxn, OEMolPropertyList trendPropFxn, OELocalKrigeOptions arg4) -> OEMolKrige
        __init__(OEStats::OEMolKrige self, OEMolKrige rhs) -> OEMolKrige
        """
        this = _oestats.new_OEMolKrige(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _oestats.delete_OEMolKrige
    __del__ = lambda self: None
OEMolKrige_swigregister = _oestats.OEMolKrige_swigregister
OEMolKrige_swigregister(OEMolKrige)

class OEMolKrigeDistFxn(object):
    """Proxy of C++ OESystem::OEBinaryFunction<(OEChem::OEMolBase,OEChem::OEMolBase,double,true,true)> class."""

    thisown = _swig_property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
    __repr__ = _swig_repr
    __swig_destroy__ = _oestats.delete_OEMolKrigeDistFxn
    __del__ = lambda self: None

    def __call__(self, a1: 'OEMolBase', a2: 'OEMolBase') -> "double":
        """__call__(OEMolKrigeDistFxn self, OEMolBase a1, OEMolBase a2) -> double"""
        return _oestats.OEMolKrigeDistFxn___call__(self, a1, a2)


    def CreateCopy(self, *args):
        try:
            return _oestats.OEMolKrigeDistFxn_CreateCopy(self, *args)
        except RuntimeError:
            try:
                return self.__class__().__disown__()
            except:
                raise RuntimeError("CreateCopy must be specialized for %s" % self.__class__.__name__)



    def __init__(self):
        """__init__(OESystem::OEBinaryFunction<(OEChem::OEMolBase,OEChem::OEMolBase,double,true,true)> self) -> OEMolKrigeDistFxn"""
        if self.__class__ == OEMolKrigeDistFxn:
            _self = None
        else:
            _self = self
        this = _oestats.new_OEMolKrigeDistFxn(_self, )
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    def __disown__(self):
        self.this.disown()
        _oestats.disown_OEMolKrigeDistFxn(self)
        return weakref_proxy(self)
OEMolKrigeDistFxn_swigregister = _oestats.OEMolKrigeDistFxn_swigregister
OEMolKrigeDistFxn_swigregister(OEMolKrigeDistFxn)

class OEMolKrigeDefaultDistFxn(OEMolKrigeDistFxn):
    """Proxy of C++ OEStats::OEMolKrigeDefaultDistFxn class."""

    thisown = _swig_property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
    __repr__ = _swig_repr

    def __call__(self, arg2: 'OEMolBase', arg3: 'OEMolBase') -> "double":
        """__call__(OEMolKrigeDefaultDistFxn self, OEMolBase arg2, OEMolBase arg3) -> double"""
        return _oestats.OEMolKrigeDefaultDistFxn___call__(self, arg2, arg3)


    def PrepMol(self, arg2: 'OEMolBase') -> "bool":
        """PrepMol(OEMolKrigeDefaultDistFxn self, OEMolBase arg2) -> bool"""
        return _oestats.OEMolKrigeDefaultDistFxn_PrepMol(self, arg2)


    def CreateCopy(self, *args):
        try:
            return _oestats.OEMolKrigeDefaultDistFxn_CreateCopy(self, *args)
        except RuntimeError:
            try:
                return self.__class__().__disown__()
            except:
                raise RuntimeError("CreateCopy must be specialized for %s" % self.__class__.__name__)



    def __init__(self, *args):
        """
        __init__(OEStats::OEMolKrigeDefaultDistFxn self, OEDefaultKrigeDistanceOptions arg2) -> OEMolKrigeDefaultDistFxn
        __init__(OEStats::OEMolKrigeDefaultDistFxn self) -> OEMolKrigeDefaultDistFxn
        """
        this = _oestats.new_OEMolKrigeDefaultDistFxn(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _oestats.delete_OEMolKrigeDefaultDistFxn
    __del__ = lambda self: None
OEMolKrigeDefaultDistFxn_swigregister = _oestats.OEMolKrigeDefaultDistFxn_swigregister
OEMolKrigeDefaultDistFxn_swigregister(OEMolKrigeDefaultDistFxn)

class OEMolKrigeGraphSimDistFxn(OEMolKrigeDistFxn):
    """Proxy of C++ OEStats::OEMolKrigeGraphSimDistFxn class."""

    thisown = _swig_property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
    __repr__ = _swig_repr

    def __call__(self, arg2: 'OEMolBase', arg3: 'OEMolBase') -> "double":
        """__call__(OEMolKrigeGraphSimDistFxn self, OEMolBase arg2, OEMolBase arg3) -> double"""
        return _oestats.OEMolKrigeGraphSimDistFxn___call__(self, arg2, arg3)


    def PrepMol(self, arg2: 'OEMolBase') -> "bool":
        """PrepMol(OEMolKrigeGraphSimDistFxn self, OEMolBase arg2) -> bool"""
        return _oestats.OEMolKrigeGraphSimDistFxn_PrepMol(self, arg2)


    def CreateCopy(self, *args):
        try:
            return _oestats.OEMolKrigeGraphSimDistFxn_CreateCopy(self, *args)
        except RuntimeError:
            try:
                return self.__class__().__disown__()
            except:
                raise RuntimeError("CreateCopy must be specialized for %s" % self.__class__.__name__)



    def __init__(self, *args):
        """
        __init__(OEStats::OEMolKrigeGraphSimDistFxn self, unsigned int fpType) -> OEMolKrigeGraphSimDistFxn
        __init__(OEStats::OEMolKrigeGraphSimDistFxn self) -> OEMolKrigeGraphSimDistFxn
        """
        this = _oestats.new_OEMolKrigeGraphSimDistFxn(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _oestats.delete_OEMolKrigeGraphSimDistFxn
    __del__ = lambda self: None
OEMolKrigeGraphSimDistFxn_swigregister = _oestats.OEMolKrigeGraphSimDistFxn_swigregister
OEMolKrigeGraphSimDistFxn_swigregister(OEMolKrigeGraphSimDistFxn)

class OEKrigeReportOptions(object):
    """Proxy of C++ OEStats::OEKrigeReportOptions class."""

    thisown = _swig_property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
    __repr__ = _swig_repr

    def SetTitle(self, title: 'std::string') -> "void":
        """SetTitle(OEKrigeReportOptions self, std::string title)"""
        return _oestats.OEKrigeReportOptions_SetTitle(self, title)


    def SetResponseName(self, name: 'std::string') -> "void":
        """SetResponseName(OEKrigeReportOptions self, std::string name)"""
        return _oestats.OEKrigeReportOptions_SetResponseName(self, name)


    def SetResponseSignificantFigures(self, arg2: 'unsigned int') -> "void":
        """SetResponseSignificantFigures(OEKrigeReportOptions self, unsigned int arg2)"""
        return _oestats.OEKrigeReportOptions_SetResponseSignificantFigures(self, arg2)


    def SetShowConfidence(self, show: 'bool') -> "void":
        """SetShowConfidence(OEKrigeReportOptions self, bool show)"""
        return _oestats.OEKrigeReportOptions_SetShowConfidence(self, show)


    def SetUnlog(self, unlog: 'bool') -> "void":
        """SetUnlog(OEKrigeReportOptions self, bool unlog)"""
        return _oestats.OEKrigeReportOptions_SetUnlog(self, unlog)


    def SetContributionPositiveColor(self, arg2: 'OEColor') -> "void":
        """SetContributionPositiveColor(OEKrigeReportOptions self, OEColor arg2)"""
        return _oestats.OEKrigeReportOptions_SetContributionPositiveColor(self, arg2)


    def SetContributionNegativeColor(self, arg2: 'OEColor') -> "void":
        """SetContributionNegativeColor(OEKrigeReportOptions self, OEColor arg2)"""
        return _oestats.OEKrigeReportOptions_SetContributionNegativeColor(self, arg2)


    def SetContributionPositiveLabel(self, arg2: 'std::string') -> "void":
        """SetContributionPositiveLabel(OEKrigeReportOptions self, std::string arg2)"""
        return _oestats.OEKrigeReportOptions_SetContributionPositiveLabel(self, arg2)


    def SetContributionNegativeLabel(self, arg2: 'std::string') -> "void":
        """SetContributionNegativeLabel(OEKrigeReportOptions self, std::string arg2)"""
        return _oestats.OEKrigeReportOptions_SetContributionNegativeLabel(self, arg2)


    def SetContributionBaseline(self, baseline: 'double') -> "void":
        """SetContributionBaseline(OEKrigeReportOptions self, double baseline)"""
        return _oestats.OEKrigeReportOptions_SetContributionBaseline(self, baseline)


    def PushSDProperty(self, *args) -> "void":
        """
        PushSDProperty(OEKrigeReportOptions self, std::string tag)
        PushSDProperty(OEKrigeReportOptions self, std::string tag, std::string name)
        """
        return _oestats.OEKrigeReportOptions_PushSDProperty(self, *args)


    def PushProperty(self, arg2: 'OEMolPropertyFxn', name: 'std::string', sigFigs: 'unsigned int'=4) -> "void":
        """
        PushProperty(OEKrigeReportOptions self, OEMolPropertyFxn arg2, std::string name, unsigned int sigFigs=4)
        PushProperty(OEKrigeReportOptions self, OEMolPropertyFxn arg2, std::string name)
        """
        return _oestats.OEKrigeReportOptions_PushProperty(self, arg2, name, sigFigs)


    def PushClassificationBoundry(self, value: 'double') -> "void":
        """PushClassificationBoundry(OEKrigeReportOptions self, double value)"""
        return _oestats.OEKrigeReportOptions_PushClassificationBoundry(self, value)


    def PushClassificationColor(self, color: 'OEColor') -> "void":
        """PushClassificationColor(OEKrigeReportOptions self, OEColor color)"""
        return _oestats.OEKrigeReportOptions_PushClassificationColor(self, color)


    def SetShowKrigeDetails(self, show: 'bool') -> "void":
        """SetShowKrigeDetails(OEKrigeReportOptions self, bool show)"""
        return _oestats.OEKrigeReportOptions_SetShowKrigeDetails(self, show)


    def GetTitle(self) -> "std::string":
        """GetTitle(OEKrigeReportOptions self) -> std::string"""
        return _oestats.OEKrigeReportOptions_GetTitle(self)


    def GetResponseName(self) -> "std::string":
        """GetResponseName(OEKrigeReportOptions self) -> std::string"""
        return _oestats.OEKrigeReportOptions_GetResponseName(self)


    def GetResponseSignificantFigures(self) -> "unsigned int":
        """GetResponseSignificantFigures(OEKrigeReportOptions self) -> unsigned int"""
        return _oestats.OEKrigeReportOptions_GetResponseSignificantFigures(self)


    def GetShowConfidence(self) -> "bool":
        """GetShowConfidence(OEKrigeReportOptions self) -> bool"""
        return _oestats.OEKrigeReportOptions_GetShowConfidence(self)


    def GetUnlog(self) -> "bool":
        """GetUnlog(OEKrigeReportOptions self) -> bool"""
        return _oestats.OEKrigeReportOptions_GetUnlog(self)


    def GetContributionPositiveColor(self) -> "OESystem::OEColor":
        """GetContributionPositiveColor(OEKrigeReportOptions self) -> OEColor"""
        return _oestats.OEKrigeReportOptions_GetContributionPositiveColor(self)


    def GetContributionNegativeColor(self) -> "OESystem::OEColor":
        """GetContributionNegativeColor(OEKrigeReportOptions self) -> OEColor"""
        return _oestats.OEKrigeReportOptions_GetContributionNegativeColor(self)


    def GetContributionPositiveLabel(self) -> "std::string":
        """GetContributionPositiveLabel(OEKrigeReportOptions self) -> std::string"""
        return _oestats.OEKrigeReportOptions_GetContributionPositiveLabel(self)


    def GetContributionNegativeLabel(self) -> "std::string":
        """GetContributionNegativeLabel(OEKrigeReportOptions self) -> std::string"""
        return _oestats.OEKrigeReportOptions_GetContributionNegativeLabel(self)


    def GetHasContributionBaseline(self) -> "bool":
        """GetHasContributionBaseline(OEKrigeReportOptions self) -> bool"""
        return _oestats.OEKrigeReportOptions_GetHasContributionBaseline(self)


    def GetContributionBaseline(self) -> "double":
        """GetContributionBaseline(OEKrigeReportOptions self) -> double"""
        return _oestats.OEKrigeReportOptions_GetContributionBaseline(self)


    def GetNumProperties(self) -> "unsigned int":
        """GetNumProperties(OEKrigeReportOptions self) -> unsigned int"""
        return _oestats.OEKrigeReportOptions_GetNumProperties(self)


    def GetPropertyName(self, iProp: 'unsigned int') -> "std::string":
        """GetPropertyName(OEKrigeReportOptions self, unsigned int iProp) -> std::string"""
        return _oestats.OEKrigeReportOptions_GetPropertyName(self, iProp)


    def GetPropertyValue(self, *args) -> "std::string":
        """
        GetPropertyValue(OEKrigeReportOptions self, unsigned int iProp, OEMolBase arg3) -> std::string
        GetPropertyValue(OEKrigeReportOptions self, unsigned int iProp, OEMCMolBase arg3) -> std::string
        """
        return _oestats.OEKrigeReportOptions_GetPropertyValue(self, *args)


    def GetNumClassifications(self) -> "unsigned int":
        """GetNumClassifications(OEKrigeReportOptions self) -> unsigned int"""
        return _oestats.OEKrigeReportOptions_GetNumClassifications(self)


    def GetClassificationColor(self, iClass: 'unsigned int') -> "OESystem::OEColor":
        """GetClassificationColor(OEKrigeReportOptions self, unsigned int iClass) -> OEColor"""
        return _oestats.OEKrigeReportOptions_GetClassificationColor(self, iClass)


    def GetClassificationUpperBound(self, iClass: 'unsigned int') -> "double":
        """GetClassificationUpperBound(OEKrigeReportOptions self, unsigned int iClass) -> double"""
        return _oestats.OEKrigeReportOptions_GetClassificationUpperBound(self, iClass)


    def GetClassificationLowerBound(self, iClass: 'unsigned int') -> "double":
        """GetClassificationLowerBound(OEKrigeReportOptions self, unsigned int iClass) -> double"""
        return _oestats.OEKrigeReportOptions_GetClassificationLowerBound(self, iClass)


    def GetShowKrigeDetails(self) -> "bool":
        """GetShowKrigeDetails(OEKrigeReportOptions self) -> bool"""
        return _oestats.OEKrigeReportOptions_GetShowKrigeDetails(self)


    def ClearTitle(self) -> "void":
        """ClearTitle(OEKrigeReportOptions self)"""
        return _oestats.OEKrigeReportOptions_ClearTitle(self)


    def ClearContributionLabels(self) -> "void":
        """ClearContributionLabels(OEKrigeReportOptions self)"""
        return _oestats.OEKrigeReportOptions_ClearContributionLabels(self)


    def ClearContributionBaseline(self) -> "void":
        """ClearContributionBaseline(OEKrigeReportOptions self)"""
        return _oestats.OEKrigeReportOptions_ClearContributionBaseline(self)


    def ClearProperties(self) -> "void":
        """ClearProperties(OEKrigeReportOptions self)"""
        return _oestats.OEKrigeReportOptions_ClearProperties(self)


    def ClearClassifications(self) -> "void":
        """ClearClassifications(OEKrigeReportOptions self)"""
        return _oestats.OEKrigeReportOptions_ClearClassifications(self)


    def Clear(self) -> "void":
        """Clear(OEKrigeReportOptions self)"""
        return _oestats.OEKrigeReportOptions_Clear(self)


    def __init__(self, *args):
        """
        __init__(OEStats::OEKrigeReportOptions self) -> OEKrigeReportOptions
        __init__(OEStats::OEKrigeReportOptions self, OEKrigeReportOptions arg2) -> OEKrigeReportOptions
        """
        this = _oestats.new_OEKrigeReportOptions(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _oestats.delete_OEKrigeReportOptions
    __del__ = lambda self: None
OEKrigeReportOptions_swigregister = _oestats.OEKrigeReportOptions_swigregister
OEKrigeReportOptions_swigregister(OEKrigeReportOptions)


def OEKrigeRenderReport(*args) -> "bool":
    """
    OEKrigeRenderReport(OEDepict::OEImageBase & image, OEMolKrigeResult result, OEMolBase mol, OEKrigeReportOptions opts, unsigned int trainRows=0, unsigned int trainCols=0) -> bool
    OEKrigeRenderReport(OEDepict::OEImageBase & image, OEMolKrigeResult result, OEMolBase mol, OEKrigeReportOptions opts, unsigned int trainRows=0) -> bool
    OEKrigeRenderReport(OEDepict::OEImageBase & image, OEMolKrigeResult result, OEMolBase mol, OEKrigeReportOptions opts) -> bool
    OEKrigeRenderReport(OEDepict::OEImageBase & image, OEMolKrigeResult result, OEMCMolBase mol, OEKrigeReportOptions opts, unsigned int trainRows=0, unsigned int trainCols=0) -> bool
    OEKrigeRenderReport(OEDepict::OEImageBase & image, OEMolKrigeResult result, OEMCMolBase mol, OEKrigeReportOptions opts, unsigned int trainRows=0) -> bool
    OEKrigeRenderReport(OEDepict::OEImageBase & image, OEMolKrigeResult result, OEMCMolBase mol, OEKrigeReportOptions opts) -> bool
    OEKrigeRenderReport(std::string filename, OEMolKrigeResult result, OEMolBase mol, OEKrigeReportOptions opts) -> bool
    OEKrigeRenderReport(std::string filename, OEMolKrigeResult result, OEMolBase mol) -> bool
    OEKrigeRenderReport(std::string filename, OEMolKrigeResult result, OEMCMolBase mol, OEKrigeReportOptions opts) -> bool
    OEKrigeRenderReport(std::string filename, OEMolKrigeResult result, OEMCMolBase mol) -> bool
    """
    return _oestats.OEKrigeRenderReport(*args)
class OEMolLog10PropertyFxn(oechem.OEMolPropertyFxn):
    """Proxy of C++ OEStats::OEMolLog10PropertyFxn class."""

    thisown = _swig_property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
    __repr__ = _swig_repr

    def __init__(self, *args):
        """
        __init__(OEStats::OEMolLog10PropertyFxn self, OEMolPropertyFxn fxn) -> OEMolLog10PropertyFxn
        __init__(OEStats::OEMolLog10PropertyFxn self, OEMolLog10PropertyFxn prop) -> OEMolLog10PropertyFxn
        """
        this = _oestats.new_OEMolLog10PropertyFxn(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this

    def Has(self, *args) -> "bool":
        """
        Has(OEMolLog10PropertyFxn self, OEMolBase mol) -> bool
        Has(OEMolLog10PropertyFxn self, OEMCMolBase mol) -> bool
        """
        return _oestats.OEMolLog10PropertyFxn_Has(self, *args)


    def Get(self, *args) -> "double":
        """
        Get(OEMolLog10PropertyFxn self, OEMolBase mol) -> double
        Get(OEMolLog10PropertyFxn self, OEMCMolBase mol) -> double
        """
        return _oestats.OEMolLog10PropertyFxn_Get(self, *args)


    def CreateCopy(self, *args):
        try:
            return _oestats.OEMolLog10PropertyFxn_CreateCopy(self, *args)
        except RuntimeError:
            try:
                return self.__class__().__disown__()
            except:
                raise RuntimeError("CreateCopy must be specialized for %s" % self.__class__.__name__)


    __swig_destroy__ = _oestats.delete_OEMolLog10PropertyFxn
    __del__ = lambda self: None
OEMolLog10PropertyFxn_swigregister = _oestats.OEMolLog10PropertyFxn_swigregister
OEMolLog10PropertyFxn_swigregister(OEMolLog10PropertyFxn)

class OEMolecularWeightPropertyFxn(oechem.OEMolPropertyFxn):
    """Proxy of C++ OEStats::OEMolecularWeightPropertyFxn class."""

    thisown = _swig_property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc='The membership flag')
    __repr__ = _swig_repr

    def Has(self, *args) -> "bool":
        """
        Has(OEMolecularWeightPropertyFxn self, OEMolBase mol) -> bool
        Has(OEMolecularWeightPropertyFxn self, OEMCMolBase mol) -> bool
        """
        return _oestats.OEMolecularWeightPropertyFxn_Has(self, *args)


    def Get(self, *args) -> "double":
        """
        Get(OEMolecularWeightPropertyFxn self, OEMolBase mol) -> double
        Get(OEMolecularWeightPropertyFxn self, OEMCMolBase mol) -> double
        """
        return _oestats.OEMolecularWeightPropertyFxn_Get(self, *args)


    def CreateCopy(self, *args):
        try:
            return _oestats.OEMolecularWeightPropertyFxn_CreateCopy(self, *args)
        except RuntimeError:
            try:
                return self.__class__().__disown__()
            except:
                raise RuntimeError("CreateCopy must be specialized for %s" % self.__class__.__name__)



    def __init__(self):
        """__init__(OEStats::OEMolecularWeightPropertyFxn self) -> OEMolecularWeightPropertyFxn"""
        this = _oestats.new_OEMolecularWeightPropertyFxn()
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _oestats.delete_OEMolecularWeightPropertyFxn
    __del__ = lambda self: None
OEMolecularWeightPropertyFxn_swigregister = _oestats.OEMolecularWeightPropertyFxn_swigregister
OEMolecularWeightPropertyFxn_swigregister(OEMolecularWeightPropertyFxn)

OE_OESTATS_VERSION = _oestats.OE_OESTATS_VERSION

def OEStatsGetRelease() -> "char const *":
    """OEStatsGetRelease() -> char const *"""
    return _oestats.OEStatsGetRelease()

def OEStatsGetVersion() -> "unsigned int":
    """OEStatsGetVersion() -> unsigned int"""
    return _oestats.OEStatsGetVersion()

def OEStatsGetArch() -> "char const *":
    """OEStatsGetArch() -> char const *"""
    return _oestats.OEStatsGetArch()

def OEStatsGetPlatform() -> "char const *":
    """OEStatsGetPlatform() -> char const *"""
    return _oestats.OEStatsGetPlatform()

def OEStatsIsLicensed(feature: 'char const *'=None, expdate: 'unsigned int *'=None) -> "bool":
    """
    OEStatsIsLicensed(char const * feature=None, unsigned int * expdate=None) -> bool
    OEStatsIsLicensed(char const * feature=None) -> bool
    OEStatsIsLicensed() -> bool
    """
    return _oestats.OEStatsIsLicensed(feature, expdate)

def OEStatsGetLicensee(*args) -> "std::string":
    """
    OEStatsGetLicensee(std::string & licensee) -> bool
    OEStatsGetLicensee() -> std::string
    """
    return _oestats.OEStatsGetLicensee(*args)

def OEStatsGetSite(*args) -> "std::string":
    """
    OEStatsGetSite(std::string & site) -> bool
    OEStatsGetSite() -> std::string
    """
    return _oestats.OEStatsGetSite(*args)





#!/opt/anaconda1anaconda2anaconda3/bin/python
# (C) 2017 OpenEye Scientific Software Inc. All rights reserved.
#
# TERMS FOR USE OF SAMPLE CODE The software below ("Sample Code") is
# provided to current licensees or subscribers of OpenEye products or
# SaaS offerings (each a "Customer").
# Customer is hereby permitted to use, copy, and modify the Sample Code,
# subject to these terms. OpenEye claims no rights to Customer's
# modifications. Modification of Sample Code is at Customer's sole and
# exclusive risk. Sample Code may require Customer to have a then
# current license or subscription to the applicable OpenEye offering.
# THE SAMPLE CODE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED.  OPENEYE DISCLAIMS ALL WARRANTIES, INCLUDING, BUT
# NOT LIMITED TO, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE AND NONINFRINGEMENT. In no event shall OpenEye be
# liable for any damages or liability in connection with the Sample Code
# or its use.

import sys
import os
import socket

try:
    from SocketServer import ThreadingMixIn
except ImportError:
    from socketserver import ThreadingMixIn
from threading import Thread
from threading import Lock
from threading import Condition
from threading import Event

from openeye import oechem
from openeye import oeshape

try:
    from openeye import oefastrocs
except ImportError:
    oechem.OEThrow.Fatal("This script is not available, "
                         "FastROCS is not supported on this platform.")

try:
    from xmlrpclib import Binary
    from SimpleXMLRPCServer import SimpleXMLRPCServer, SimpleXMLRPCRequestHandler
except ImportError:  # python 3
    from xmlrpc.client import Binary
    from xmlrpc.server import SimpleXMLRPCServer, SimpleXMLRPCRequestHandler

oepy = os.path.join(os.path.dirname(__file__), "..", "python")
sys.path.insert(0, os.path.realpath(oepy))

# very important that OEChem is in this mode since we are passing molecules between threads
oechem.OESetMemPoolMode(oechem.OEMemPoolMode_System)


class ReadWriteLock(object):
    """ Basic locking primitive that allows multiple readers but only
    a single writer at a time. Useful for synchronizing database
    updates. Priority is given to pending writers. """
    def __init__(self):
        self.cond = Condition()
        self.readers = 0
        self.writers = 0

    def AcquireReadLock(self):
        self.cond.acquire()
        try:
            while self.writers:
                self.cond.wait()

            self.readers += 1
            assert self.writers == 0
        finally:
            self.cond.notifyAll()
            self.cond.release()

    def ReleaseReadLock(self):
        self.cond.acquire()
        assert self.readers > 0
        try:
            self.readers -= 1
        finally:
            self.cond.notifyAll()
            self.cond.release()

    def AcquireWriteLock(self):
        self.cond.acquire()
        self.writers += 1
        while self.readers:
            self.cond.wait()

        assert self.readers == 0
        assert self.writers > 0

    def ReleaseWriteLock(self):
        assert self.readers == 0
        assert self.writers > 0

        self.writers -= 1
        self.cond.notifyAll()
        self.cond.release()


class ShapeQueryThread(Thread):
    """ A thread to run a query against a shape database """

    def __init__(self, shapedb, querymolstr, nhits, iformat, oformat, errorLevel, **kwargs):
        """ Create a new thread to perform a query. The query doesn't
        execute until start is called.
        shapedb - database to run the query against

        See MCMolShapeDatabase.GetBestOverlays for a description of
        the querymolstr and nhits arguments.
        """
        Thread.__init__(self)

        self.shapeOnly = kwargs.pop('shapeOnly', False)
        self.tversky = kwargs.pop('tversky', False)
        self.altStarts = kwargs.pop('altStarts', False)
        self.randStarts = kwargs.pop('randStarts', False)

        self.shapedb = shapedb
        self.querymolstr = querymolstr
        self.iformat = iformat
        self.oformat = oformat
        self.scoretype = GetDatabaseType(self.shapeOnly)
        self.simFuncType = GetSimFuncType(self.tversky)

        numHistBins = 200
        if self.shapeOnly:
            numHistBins = 100
        self.tracer = oefastrocs.OEDBTracer(numHistBins)
        self.options = oefastrocs.OEShapeDatabaseOptions()
        self.options.SetTracer(self.tracer)
        self.options.SetLimit(nhits)
        self.options.SetScoreType(self.scoretype)
        self.options.SetSimFunc(self.simFuncType)

        if self.altStarts:
            self.options.SetInitialOrientation(GetStartType(self.altStarts))
            if self.randStarts:
                self.options.SetNumRandomStarts(self.randStarts)

        self.lock = Lock()
        self.errorLevel = errorLevel

    def run(self):
        """ Perform the query """
        # make sure the error level is set for this operating system thread
        oechem.OEThrow.SetLevel(self.errorLevel)
        try:
            results = self.shapedb.GetBestOverlays(self.querymolstr,
                                                   self.options,
                                                   self.iformat,
                                                   self.oformat)

            # since we are writing to the thread's dictionary this could
            # race with the GetStatus method below
            self.lock.acquire()
            try:
                self.results = results
                if not results:
                    self.exception = RuntimeError("Query error, no results to return, "
                                                  "check the server log for more information")
            finally:
                self.lock.release()

        except Exception as e:
            self.lock.acquire()
            try:
                self.exception = e
            finally:
                self.lock.release()

    def GetStatus(self, blocking):
        """ Returns a tuple of (count, total). count is the number of
        conformers already searched. total is the total number of
        conformers that will be searched.

        If blocking is True this method will not return until the
        count has been changed (beware of deadlocks!). If blocking is
        False the function will return immediately.
        """
        self.lock.acquire()
        try:
            if hasattr(self, "exception"):
                raise self.exception

            return self.tracer.GetCounts(blocking), self.tracer.GetTotal()
        finally:
            self.lock.release()

    def GetHistogram(self):
        """ Returns a list of integers representing the histogram of
        the molecule scores already scored.
        """
        self.lock.acquire()
        try:
            if hasattr(self, "exception"):
                raise self.exception

            hist = self.tracer.GetHistogram()
            scoretype = self.scoretype
        finally:
            self.lock.release()

        frequencies = oechem.OEUIntVector()
        hist.GetHistogram(frequencies, scoretype)
        return list(frequencies)

    def GetResults(self):
        """ Return an OEB string containing the overlaid
        confomers. This method should only be called after this thread
        has been joined. """

        if hasattr(self, "exception"):
            raise self.exception

        return self.results


class ShapeQueryThreadPool:
    """
    Maintains a pool of threads querying the same MCMolShapeDatabase.
    """
    def __init__(self, dbase):
        """ Create a new thread pool to issues queries to dbase """
        self.shapedb = dbase
        self.queryidx = 0
        self.threads = {}
        self.lock = Lock()
        self.errorLevel = oechem.OEThrow.GetLevel()

    def SubmitQuery(self, querymolstr, nhits, iformat, oformat, kwargs):
        """ Returns an index that can be passed to the QueryStatus and
        QueryResults methods.

        See MCMolShapeDatabase.GetBestOverlays for a description of
        the querymolstr and nhits arguments.
        """
        self.lock.acquire()
        try:
            idx = self.queryidx
            self.queryidx += 1
            self.threads[idx] = ShapeQueryThread(self.shapedb,
                                                 querymolstr,
                                                 nhits,
                                                 iformat,
                                                 oformat,
                                                 self.errorLevel,
                                                 **kwargs)
            self.threads[idx].start()
        finally:
            self.lock.release()

        return idx

    def QueryStatus(self, idx, blocking):
        """ Returns the status of the query indicated by idx. See
        ShapeQueryThread.GetStatus for the description of the blocking
        argument. """
        self.lock.acquire()
        try:
            thrd = self.threads[idx]
        finally:
            self.lock.release()

        return thrd.GetStatus(blocking)

    def QueryHistogram(self, idx):
        """ Returns the histogram of molecule scores already scored
        for the query indicated by idx. """
        self.lock.acquire()
        try:
            thrd = self.threads[idx]
        finally:
            self.lock.release()

        return thrd.GetHistogram()

    def QueryResults(self, idx):
        """ Wait for the query associated with idx to complete and
        then return the results as an OEB string. """
        self.lock.acquire()
        try:
            thrd = self.threads[idx]
            del self.threads[idx]
        finally:
            self.lock.release()

        thrd.join()
        return thrd.GetResults()

    def SetLevel(self, level):
        """ Set what level of information should be printed by the server. """
        self.errorLevel = level
        return True


class DatabaseLoaderThread(Thread):
    """ A thread to read a database into memory. Special note, OEChem
    must be placed in system allocation mode using
    oechem.OESetMemPoolMode(oechem.OEMemPoolMode_System). This is because the
    default OEChem memory caching scheme uses thread local storage,
    but since this thread is temporary only for reading in molecules
    that memory will be deallocated when this thread is terminated."""
    def __init__(self, shapedb, moldb, dbname, loadedEvent):
        """
        shapedb - the shapedb to add the molecules to
        moldb   - the OEMolDatabase object to use
        dbname  - the file name to open the OEMolDatabase on
        loadedEvent - event to set once loading is finished
        """
        Thread.__init__(self)
        self.shapedb = shapedb
        self.moldb = moldb
        self.dbname = dbname
        self.loadedEvent = loadedEvent

    def run(self):
        """ Open the database file and load it into the OEShapeDatabase """
        timer = oechem.OEWallTimer()
        sys.stderr.write("Opening database file %s ...\n" % self.dbname)
        if not self.moldb.Open(self.dbname):
            oechem.OEThrow.Fatal("Unable to open '%s'" % self.dbname)

        dots = oechem.OEThreadedDots(10000, 200, "conformers")
        if not self.shapedb.Open(self.moldb, dots):
            oechem.OEThrow.Fatal("Unable to initialize OEShapeDatabase on '%s'" % self.dbname)

        dots.Total()
        sys.stderr.write("%s seconds to load database\n" % timer.Elapsed())
        self.loadedEvent.set()


def SetupStream(strm, format):
    format = format.strip('.')
    ftype = oechem.OEGetFileType(format)
    if ftype == oechem.OEFormat_UNDEFINED:
        raise ValueError("Unsupported file format sent to server '%s'" % format)
    strm.SetFormat(ftype)
    strm.Setgz(oechem.OEIsGZip(format))
    return strm


OECOLOR_FORCEFIELDS = {
    "ImplicitMillsDean": oeshape.OEColorFFType_ImplicitMillsDean,
    "ImplicitMillsDeanNoRings": oeshape.OEColorFFType_ImplicitMillsDeanNoRings,
    "ExplicitMillsDean": oeshape.OEColorFFType_ExplicitMillsDean,
    "ExplicitMillsDeanNoRings": oeshape.OEColorFFType_ExplicitMillsDeanNoRings
    }


def GetDatabaseType(shapeOnly):
    if shapeOnly:
        return oefastrocs.OEShapeDatabaseType_Shape
    return oefastrocs.OEShapeDatabaseType_Default


def GetSimFuncType(simFuncType):
    if simFuncType:
        return oefastrocs.OEShapeSimFuncType_Tversky
    return oefastrocs.OEShapeSimFuncType_Tanimoto


def GetStartType(altStarts):
    if altStarts == 'random':
        return oefastrocs.OEFastROCSOrientation_Random
    if altStarts == 'inertialAtHeavyAtoms':
        return oefastrocs.OEFastROCSOrientation_InertialAtHeavyAtoms
    if altStarts == 'inertialAtColorAtoms':
        return oefastrocs.OEFastROCSOrientation_InertialAtColorAtoms
    if altStarts == 'subrocs':
        return oefastrocs.OEFastROCSOrientation_Subrocs
    return oefastrocs.OEFastROCSOrientation_Inertial


def GetAltStartsString(altStarts):
    if altStarts == oefastrocs.OEFastROCSOrientation_Random:
        return 'random'
    if altStarts == oefastrocs.OEFastROCSOrientation_InertialAtHeavyAtoms:
        return 'inertialAtHeavyAtoms'
    if altStarts == oefastrocs.OEFastROCSOrientation_InertialAtColorAtoms:
        return 'inertialAtColorAtoms'
    if altStarts == oefastrocs.OEFastROCSOrientation_Subrocs:
        return 'subrocs'
    return 'inertial'


def GetShapeDatabaseArgs(itf):
    shapeOnly = itf.GetBool("-shapeOnly")
    if shapeOnly and itf.GetParameter("-chemff").GetHasValue():
        oechem.OEThrow.Fatal("Unable to specify -shapeOnly and -chemff at the same time!")

    chemff = itf.GetString("-chemff")
    if not chemff.endswith(".cff"):
        return (GetDatabaseType(shapeOnly), OECOLOR_FORCEFIELDS[chemff])

    # given a .cff file, use that to construct a OEColorForceField
    assert not shapeOnly
    cff = oeshape.OEColorForceField()
    if not cff.Init(chemff):
        oechem.OEThrow.Fatal("Unable to read color force field from '%s'" % chemff)

    return (cff,)


def ReadShapeQuery(querymolstr):
    iss = oechem.oeisstream(querymolstr)
    query = oeshape.OEShapeQueryPublic()

    if not oeshape.OEReadShapeQuery(iss, query):
        raise ValueError("Unable to read a shape query from the data string")

    return query


class MCMolShapeDatabase:
    """ Maintains a database of MCMols that can be queried by shape
    similarity."""
    def __init__(self, itf):
        """ Create a MCMolShapeDatabase from the parameters specified by the OEInterface. """
        self.rwlock = ReadWriteLock()
        self.loadedEvent = Event()

        self.dbname = itf.GetString("-dbase")
        self.moldb = oechem.OEMolDatabase()

        self.dbtype = GetDatabaseType(itf.GetBool("-shapeOnly"))
        self.shapedb = oefastrocs.OEShapeDatabase(*GetShapeDatabaseArgs(itf))

        # this thread is daemonic so a KeyboardInterupt
        # during the load will cancel the process
        self.loaderThread = DatabaseLoaderThread(self.shapedb,
                                                 self.moldb,
                                                 self.dbname,
                                                 self.loadedEvent)
        self.loaderThread.setDaemon(True)
        self.loaderThread.start()

    def IsLoaded(self, blocking=False):
        """ Return whether the server has finished loading. """
        if blocking:
            self.loadedEvent.wait()

        # clean up the load waiter thread if it's still there
        if self.loadedEvent.isSet() and self.loaderThread is not None:
            self.rwlock.AcquireWriteLock()
            try:  # typical double checked locking
                if self.loaderThread is not None:
                    self.loaderThread.join()
                    self.loaderThread = None
            finally:
                self.rwlock.ReleaseWriteLock()

        return self.loadedEvent.isSet()

    def GetBestOverlays(self, querymolstr, options, iformat, oformat):
        """ Return a string of the format specified by 'oformat'
        containing nhits overlaid confomers using querymolstr as the
        query interpretted as iformat.

        querymolstr - a string containing a molecule to use as the query
        options - an instance of OEShapeDatabaseOptions
        iformat - a string representing the file extension to parse the querymolstr as.
                  Note: old clients could be passing .sq files, so
                  iformat == '.oeb' will try to interpret the file as
                  a .sq file.
        oformat - file format to write the results as
        """
        timer = oechem.OEWallTimer()

        # make sure to wait for the load to finish
        blocking = True
        loaded = self.IsLoaded(blocking)
        assert loaded

        if iformat.startswith(".sq"):
            query = ReadShapeQuery(querymolstr)
        else:
            # read in query
            qfs = oechem.oemolistream()
            qfs = SetupStream(qfs, iformat)
            if not qfs.openstring(querymolstr):
                raise ValueError("Unable to open input molecule string")

            query = oechem.OEGraphMol()
            if not oechem.OEReadMolecule(qfs, query):
                if iformat == ".oeb":  # could be an old client trying to send a .sq file.
                    query = ReadShapeQuery(querymolstr)
                else:
                    raise ValueError("Unable to read a molecule from the string of format '%s'"
                                     % iformat)

        ofs = oechem.oemolostream()
        ofs = SetupStream(ofs, oformat)
        if not ofs.openstring():
            raise ValueError("Unable to openstring for output")

        # do we only want shape based results?

        # this is a "Write" lock to be paranoid and not overload the GPU
        self.rwlock.AcquireWriteLock()
        try:
            # do search
            scores = self.shapedb.GetSortedScores(query, options)
            sys.stderr.write("%f seconds to do search\n" % timer.Elapsed())
        finally:
            self.rwlock.ReleaseWriteLock()

        timer.Start()
        # write results
        for score in scores:
            mcmol = oechem.OEMol()
            if not self.moldb.GetMolecule(mcmol, score.GetMolIdx()):
                oechem.OEThrow.Warning("Can't retrieve molecule %i from the OEMolDatabase, "
                                       "skipping..." % score.GetMolIdx())
                continue
            # remove hydrogens to make output smaller, this also
            # ensures OEPrepareFastROCSMol will have the same output
            oechem.OESuppressHydrogens(mcmol)

            mol = oechem.OEGraphMol(mcmol.GetConf(oechem.OEHasConfIdx(score.GetConfIdx())))
            oechem.OECopySDData(mol, mcmol)

            if options.GetSimFunc() == oefastrocs.OEShapeSimFuncType_Tanimoto:
                oechem.OESetSDData(mol, "ShapeTanimoto", "%.4f" % score.GetShapeTanimoto())
                oechem.OESetSDData(mol, "ColorTanimoto", "%.4f" % score.GetColorTanimoto())
                oechem.OESetSDData(mol, "TanimotoCombo", "%.4f" % score.GetTanimotoCombo())
            else:
                oechem.OESetSDData(mol, "ShapeTversky", "%.4f" % score.GetShapeTversky())
                oechem.OESetSDData(mol, "ColorTversky", "%.4f" % score.GetColorTversky())
                oechem.OESetSDData(mol, "TverskyCombo", "%.4f" % score.GetTverskyCombo())

            if options.GetInitialOrientation() != oefastrocs.OEFastROCSOrientation_Inertial:
                oechem.OEAddSDData(mol, "Opt. Starting Pos.",
                                   GetAltStartsString(options.GetInitialOrientation()))

            score.Transform(mol)

            oechem.OEWriteMolecule(ofs, mol)

        output = ofs.GetString()
        sys.stderr.write("%f seconds to write hitlist\n" % timer.Elapsed())
        sys.stderr.flush()
        ofs.close()

        return output

    def GetName(self):
        self.rwlock.AcquireReadLock()
        try:
            return self.dbname
        finally:
            self.rwlock.ReleaseReadLock()

    def SetName(self, name):
        self.rwlock.AcquireWriteLock()
        try:
            self.dbname = name
        finally:
            self.rwlock.ReleaseWriteLock()


class ShapeQueryServer:
    """ This object's methods are exposed via XMLRPC. """
    def __init__(self, itf):
        """ Initialize the server to serve queries on the database
        named by dbname."""
        self.shapedb = MCMolShapeDatabase(itf)
        self.thdpool = ShapeQueryThreadPool(self.shapedb)
        self.itf = itf

    def IsLoaded(self, blocking=False):
        """ Return whether the server has finished loading. """
        return self.shapedb.IsLoaded(blocking)

    def GetBestOverlays(self, querymolstr, nhits, iformat=".oeb", oformat=".oeb"):
        """ A blocking call that only returns once the query is completed. """
        results = self.shapedb.GetBestOverlays(querymolstr.data, nhits, iformat, oformat)
        return Binary(results)

    def SubmitQuery(self, querymolstr, nhits, iformat=".oeb", oformat=".oeb", kwargs=None):
        """ Returns a index that can be used by QueryStatus and
        QueryResults. This method will return immediately."""
        if not kwargs:
            kwargs = {}
        if self.itf.GetBool("-shapeOnly"):
            kwargs['shapeOnly'] = True

        return self.thdpool.SubmitQuery(querymolstr.data, nhits, iformat, oformat, kwargs)

    def QueryStatus(self, queryidx, blocking=False):
        """ Return the status of the query specified by queryidx. See
        ShapeQueryThread.GetStatus for a description of the blocking
        argument and the return value."""
        return self.thdpool.QueryStatus(queryidx, blocking)

    def QueryHistogram(self, queryidx):
        """ Return the current histogram of scores specified by
        queryidx."""
        return self.thdpool.QueryHistogram(queryidx)

    def QueryResults(self, queryidx):
        """ Wait for the query associated with idx to complete and
        then return the results as an OEB string. """
        results = self.thdpool.QueryResults(queryidx)
        return Binary(results)

    def GetVersion(self):
        """ Returns what version of FastROCS this server is. """
        return oefastrocs.OEFastROCSGetRelease()

    def OEThrowSetLevel(self, level):
        """ Set what level of information should be printed by the server. """
        return self.thdpool.SetLevel(level)

    def GetName(self):
        """ The name of this database. By default this is the file name of the database used. """
        return self.shapedb.GetName()

    def SetName(self, name):
        """ Set a custom database name for this server. """
        self.shapedb.SetName(name)
        return True


# Restrict to a particular path.
class RequestHandler(SimpleXMLRPCRequestHandler):
    rpc_paths = ('/RPC2',)


class AsyncXMLRPCServer(ThreadingMixIn, SimpleXMLRPCServer):
    # if a shutdown request occurs through a signal force everything to terminate immediately
    daemon_threads = True
    allow_reuse_address = True


InterfaceData = """\
!BRIEF [-shapeOnly | -chemff <color forcefield>] [-hostname] [-dbase] database [[-port] 8080]
!PARAMETER -dbase
  !TYPE string
  !REQUIRED true
  !BRIEF Input database to serve
  !KEYLESS 1
!END
!PARAMETER -port
  !TYPE int
  !REQUIRED false
  !BRIEF Port number to start the XML RPC server on
  !DEFAULT 8080
  !KEYLESS 2
!END
!PARAMETER -hostname
  !TYPE string
  !DEFAULT 0.0.0.0
  !BRIEF Name of the server to bind to
!END
!PARAMETER -shapeOnly
  !ALIAS -s
  !TYPE bool
  !DEFAULT false
  !BRIEF Run FastROCS server in shape only mode, clients can also control this separately
!END
!PARAMETER -chemff
  !TYPE string
  !LEGAL_VALUE ImplicitMillsDean
  !LEGAL_VALUE ImplicitMillsDeanNoRings
  !LEGAL_VALUE ExplicitMillsDean
  !LEGAL_VALUE ExplicitMillsDeanNoRings
  !LEGAL_VALUE *.cff
  !DEFAULT ImplicitMillsDean
  !BRIEF Chemical force field. Either a constant or a filename.
!END
"""


def main(argv=[__name__]):

    if not oefastrocs.OEFastROCSIsGPUReady():
        oechem.OEThrow.Fatal("No supported GPU available to run FastROCS TK!")

    itf = oechem.OEInterface(InterfaceData, argv)

    # default hostname to bind is 0.0.0.0, to allow connections with
    # any hostname
    hostname = itf.GetString("-hostname")

    # default port number is 8080
    portnumber = itf.GetInt("-port")

    # create server
    server = AsyncXMLRPCServer((hostname, portnumber),
                               requestHandler=RequestHandler,
                               logRequests=False)
    hostname, portnumber = server.socket.getsockname()
    if hostname == "0.0.0.0":
        hostname = socket.gethostname()
    sys.stderr.write("Listening for ShapeDatabaseClient.py requests on %s:%i\n\n"
                     % (hostname, portnumber))
    sys.stderr.write("Example: ShapeDatabaseClient.py %s:%i query.sdf hit.sdf\n\n"
                     % (hostname, portnumber))

    # register the XMLRPC methods
    server.register_introspection_functions()

    server.register_instance(ShapeQueryServer(itf))

    try:
        # Run the server's main loop
        server.serve_forever()
    finally:
        server.server_close()

    return 0


if __name__ == '__main__':
    sys.exit(main(sys.argv))



#!/usr/bin/env python
# (C) 2017 OpenEye Scientific Software Inc. All rights reserved.

# TERMS FOR USE OF SAMPLE CODE The software below ("Sample Code") is
# provided to current licensees or subscribers of OpenEye products or
# SaaS offerings (each a "Customer").
# Customer is hereby permitted to use, copy, and modify the Sample Code,
# subject to these terms. OpenEye claims no rights to Customer's
# modifications. Modification of Sample Code is at Customer's sole and
# exclusive risk. Sample Code may require Customer to have a then
# current license or subscription to the applicable OpenEye offering.
# THE SAMPLE CODE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED.  OPENEYE DISCLAIMS ALL WARRANTIES, INCLUDING, BUT
# NOT LIMITED TO, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE AND NONINFRINGEMENT. In no event shall OpenEye be
# liable for any damages or liability in connection with the Sample Code
# or its use.

import sys
import math
from openeye import oechem

if len(sys.argv) != 3:
    oechem.OEThrow.Usage("{} <protein> <ligand>".format(sys.argv[0]))

prot = oechem.OEGraphMol()
pifs = oechem.oemolistream(sys.argv[1])
if not oechem.OEReadMolecule(pifs, prot):
    oechem.OEThrow.Fatal("Cannot read protein from file %s!", sys.argv[1])

lig = oechem.OEGraphMol()
lifs = oechem.oemolistream(sys.argv[2])
if not oechem.OEReadMolecule(lifs, lig):
    oechem.OEThrow.Fatal("Cannot read ligand from file %s!", sys.argv[2])

asite = oechem.OEInteractionHintContainer(prot, lig)
if not oechem.OEIsValidActiveSite(asite):
    oechem.OEThrow.Fatal("Cannot initialize active site!")

# @ <SNIPPET-INTER>
oechem.OEPerceiveInteractionHints(asite)
itype = oechem.OEHBondInteractionHint(oechem.OEHBondInteractionHintType_LigandDonates)
ipred = oechem.OEHasInteractionHintType(itype)
for inter in asite.GetInteractions(ipred):
    itype = inter.GetInteractionType()
    print("Calculated geometries for '{}' interaction type:".format(itype.GetName()))
    for gname in inter.GetCalculatedGeometries():
        print("{0:30} : {1:.3f}".format(gname, inter.GetCalculatedGeometry(gname)))
# @ </SNIPPET-INTER>
print("")

iter = asite.GetInteractions(ipred)
inter = iter.Target()
# @ <SNIPPET-HAS-GEOMETRY>
if inter.HasCalculatedGeometry("distance"):
    distance = inter.GetCalculatedGeometry("distance")
    print("distance: {:.3f}  Angstrom(s)".format(distance))
if inter.HasCalculatedGeometry("acceptor-angle"):
    angle = math.degrees(inter.GetCalculatedGeometry("acceptor-angle"))
    print("acceptor angle: {:.3f} degree(s)".format(angle))
# @ </SNIPPET-HAS-GEOMETRY>
print("")

# @ <SNIPPET-INTER-TYPE>
hbondacc = oechem.OEHBondInteractionHint(oechem.OEHBondInteractionHintType_LigandAccepts)
print("Calculated geometries for '{}' interaction type:".format(hbondacc.GetName()))
for geom in oechem.OEGetCalculatedInteractionGeometries(hbondacc):
    print(geom)
#  </SNIPPET-INTER-TYPE>



#!/usr/bin/env python
# (C) 2017 OpenEye Scientific Software Inc. All rights reserved.
#
# TERMS FOR USE OF SAMPLE CODE The software below ("Sample Code") is
# provided to current licensees or subscribers of OpenEye products or
# SaaS offerings (each a "Customer").
# Customer is hereby permitted to use, copy, and modify the Sample Code,
# subject to these terms. OpenEye claims no rights to Customer's
# modifications. Modification of Sample Code is at Customer's sole and
# exclusive risk. Sample Code may require Customer to have a then
# current license or subscription to the applicable OpenEye offering.
# THE SAMPLE CODE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED.  OPENEYE DISCLAIMS ALL WARRANTIES, INCLUDING, BUT
# NOT LIMITED TO, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE AND NONINFRINGEMENT. In no event shall OpenEye be
# liable for any damages or liability in connection with the Sample Code
# or its use.

import sys

from openeye import oechem
from openeye import oedocking


def main(argv):
    if len(argv) != 3:
        oechem.OEThrow.Usage("MakeReceptorWithLigand.py <protein> <bound ligand>")

    proteinMolStream = oechem.oemolistream(argv[1])
    ligandMolStream = oechem.oemolistream(argv[2])

    # @ <SNIPPET-MAKE-RECEPTOR-WITH-LIGAND-1>
    protein = oechem.OEGraphMol()
    oechem.OEReadMolecule(proteinMolStream, protein)

    ligand = oechem.OEGraphMol()
    oechem.OEReadMolecule(ligandMolStream, ligand)

    receptor = oechem.OEGraphMol()
    oedocking.OEMakeReceptor(receptor, protein, ligand)
    # @ </SNIPPET-MAKE-RECEPTOR-WITH-LIGAND-1>

    oedocking.OEWriteReceptorFile(receptor, "receptor.oeb.gz")


if __name__ == "__main__":
    sys.exit(main(sys.argv))



#!/opt/anaconda1anaconda2anaconda3/bin/python
# (C) 2017 OpenEye Scientific Software Inc. All rights reserved.
#
# TERMS FOR USE OF SAMPLE CODE The software below ("Sample Code") is
# provided to current licensees or subscribers of OpenEye products or
# SaaS offerings (each a "Customer").
# Customer is hereby permitted to use, copy, and modify the Sample Code,
# subject to these terms. OpenEye claims no rights to Customer's
# modifications. Modification of Sample Code is at Customer's sole and
# exclusive risk. Sample Code may require Customer to have a then
# current license or subscription to the applicable OpenEye offering.
# THE SAMPLE CODE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED.  OPENEYE DISCLAIMS ALL WARRANTIES, INCLUDING, BUT
# NOT LIMITED TO, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE AND NONINFRINGEMENT. In no event shall OpenEye be
# liable for any damages or liability in connection with the Sample Code
# or its use.

#############################################################################
# Swap ambiguous isoelectronic residue atoms in a protein
#############################################################################
import sys
from openeye import oechem


def SwapAIERes(ifs, ofs, chainid, resname, resnum):
    mol = oechem.OEGraphMol()
    while oechem.OEReadMolecule(ifs, mol):
        if not oechem.OEHasResidues(mol):
            oechem.OEPerceiveResidues(mol, oechem.OEPreserveResInfo_All)
        hv = oechem.OEHierView(mol)
        res = hv.GetResidue(chainid, resname, resnum)
        if res.GetOEResidue().GetName() is None:
            oechem.OEThrow.Fatal("Failed to find residue")
        if not oechem.OESwapAIEResidueAtoms(mol, res):
            print("failed to swap %s %d" % (resname, resnum))
        oechem.OEWriteMolecule(ofs, mol)


if __name__ == "__main__":
    if len(sys.argv) < 5 or len(sys.argv) > 6:
        oechem.OEThrow.Usage("swapaieres.py <resname> [chain] <resnum> <infile> <outfile>\n"
                             "example: swapaieres.py ASN A 23 dhfr.pdb out.pdb\n"
                             "Chain ID required if the residue has a chain ID")

    resname = sys.argv[1]
    chainid = " "
    if len(sys.argv) == 5:
        resnum = int(sys.argv[2])
        infile = sys.argv[3]
        outfile = sys.argv[4]
    elif len(sys.argv) == 6:
        chainid = sys.argv[2]
        resnum = int(sys.argv[3])
        infile = sys.argv[4]
        outfile = sys.argv[5]

    ifs = oechem.oemolistream()
    if not ifs.open(infile):
        oechem.OEThrow.Fatal("Unable to open %s for reading" % infile)
    ofs = oechem.oemolostream()
    if not ofs.open(outfile):
        oechem.OEThrow.Fatal("Unable to open %s for writing" % outfile)
    SwapAIERes(ifs, ofs, chainid, resname, resnum)



#!/usr/bin/env python
# (C) 2017 OpenEye Scientific Software Inc. All rights reserved.
#
# TERMS FOR USE OF SAMPLE CODE The software below ("Sample Code") is
# provided to current licensees or subscribers of OpenEye products or
# SaaS offerings (each a "Customer").
# Customer is hereby permitted to use, copy, and modify the Sample Code,
# subject to these terms. OpenEye claims no rights to Customer's
# modifications. Modification of Sample Code is at Customer's sole and
# exclusive risk. Sample Code may require Customer to have a then
# current license or subscription to the applicable OpenEye offering.
# THE SAMPLE CODE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED.  OPENEYE DISCLAIMS ALL WARRANTIES, INCLUDING, BUT
# NOT LIMITED TO, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE AND NONINFRINGEMENT. In no event shall OpenEye be
# liable for any damages or liability in connection with the Sample Code
# or its use.

import sys
from openeye import oechem
from openeye import oezap

KCalsPerKT = 0.59
KCalsPerSqAngstrom = 0.010


def main(argv=[__name__]):

    if len(argv) != 2:
        oechem.OEThrow.Usage("%s <molfile>" % argv[0])

    epsin = 1.0

    zap = oezap.OEZap()
    zap.SetInnerDielectric(epsin)
    zap.SetGridSpacing(0.5)

    area = oezap.OEArea()

    mol = oechem.OEGraphMol()
    ifs = oechem.oemolistream()
    if not ifs.open(argv[1]):
        oechem.OEThrow.Fatal("Unable to open %s for reading" % argv[1])
    oechem.OEThrow.Info("%-20s %6s\n" % ("Title", "Vacuum->Water(kcal)"))
    while oechem.OEReadMolecule(ifs, mol):
        oechem.OEAssignBondiVdWRadii(mol)
        oechem.OEMMFFAtomTypes(mol)
        oechem.OEMMFF94PartialCharges(mol)
        zap.SetMolecule(mol)
        solv = zap.CalcSolvationEnergy()
        aval = area.GetArea(mol)
        oechem.OEThrow.Info("%-20s   %6.2f" % (mol.GetTitle(),
                                               KCalsPerKT*solv+KCalsPerSqAngstrom*aval))

    return 0


if __name__ == "__main__":
    sys.exit(main(sys.argv))



#!/usr/bin/env python
# (C) 2017 OpenEye Scientific Software Inc. All rights reserved.
#
# TERMS FOR USE OF SAMPLE CODE The software below ("Sample Code") is
# provided to current licensees or subscribers of OpenEye products or
# SaaS offerings (each a "Customer").
# Customer is hereby permitted to use, copy, and modify the Sample Code,
# subject to these terms. OpenEye claims no rights to Customer's
# modifications. Modification of Sample Code is at Customer's sole and
# exclusive risk. Sample Code may require Customer to have a then
# current license or subscription to the applicable OpenEye offering.
# THE SAMPLE CODE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED.  OPENEYE DISCLAIMS ALL WARRANTIES, INCLUDING, BUT
# NOT LIMITED TO, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE AND NONINFRINGEMENT. In no event shall OpenEye be
# liable for any damages or liability in connection with the Sample Code
# or its use.

import sys
from openeye import oechem
from openeye import oezap

if len(sys.argv) != 2:
    oechem.OEThrow.Usage("%s <molfile>" % sys.argv[0])

mol = oechem.OEGraphMol()
ifs = oechem.oemolistream()
if not ifs.open(sys.argv[1]):
    oechem.OEThrow.Fatal("Unable to open %s for reading", sys.argv[1])

if not oechem.OEIs3DFormat(ifs.GetFormat()):
    oechem.OEThrow.Fatal("Invalid input format: need 3D coordinates")

oechem.OEReadMolecule(ifs, mol)
oechem.OEAssignBondiVdWRadii(mol)
oechem.OEMMFFAtomTypes(mol)
oechem.OEMMFF94PartialCharges(mol)

# @ <SNIPPET-GetArea-Molecule>
area = oezap.OEArea()
a = area.GetArea(mol)
print("Molecule area =", a)
# @ </SNIPPET-GetArea-Molecule>

# @ <SNIPPET-GetArea-Atoms>
atomArea = oechem.OEFloatArray(mol.GetMaxAtomIdx())
area = oezap.OEArea()
area.GetArea(mol, atomArea)

for atom in mol.GetAtoms():
    idx = atom.GetIdx()
    print(idx, atomArea[idx])
# @ </SNIPPET-GetArea-Atoms>



#!/usr/bin/env python
# (C) 2017 OpenEye Scientific Software Inc. All rights reserved.
#
# TERMS FOR USE OF SAMPLE CODE The software below ("Sample Code") is
# provided to current licensees or subscribers of OpenEye products or
# SaaS offerings (each a "Customer").
# Customer is hereby permitted to use, copy, and modify the Sample Code,
# subject to these terms. OpenEye claims no rights to Customer's
# modifications. Modification of Sample Code is at Customer's sole and
# exclusive risk. Sample Code may require Customer to have a then
# current license or subscription to the applicable OpenEye offering.
# THE SAMPLE CODE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED.  OPENEYE DISCLAIMS ALL WARRANTIES, INCLUDING, BUT
# NOT LIMITED TO, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE AND NONINFRINGEMENT. In no event shall OpenEye be
# liable for any damages or liability in connection with the Sample Code
# or its use.

from openeye import oechem
from openeye import oespicoli

surf = oespicoli.OESurface()

# @ <SNIPPET-GetVertices>
coords = oechem.OEFloatArray(surf.GetNumVertices() * 3)
surf.GetVertices(coords)
# @ </SNIPPET-GetVertices>


# @ <SNIPPET-GetVertex>
for i in range(surf.GetNumVertices()):
    vert = surf.GetVertex(i)
# @ </SNIPPET-GetVertex>


# @ <SNIPPET-GetTriangles>
coords = oechem.OEUIntArray(surf.GetNumTriangles() * 3)
surf.GetTriangles(coords)
# @ </SNIPPET-GetTriangles>


# @ <SNIPPET-GetTriangle>
for i in range(surf.GetNumTriangles()):
    tri = surf.GetTriangle(i)
# @ </SNIPPET-GetTriangle>



#!/usr/bin/env python
# (C) 2017 OpenEye Scientific Software Inc. All rights reserved.
#
# TERMS FOR USE OF SAMPLE CODE The software below ("Sample Code") is
# provided to current licensees or subscribers of OpenEye products or
# SaaS offerings (each a "Customer").
# Customer is hereby permitted to use, copy, and modify the Sample Code,
# subject to these terms. OpenEye claims no rights to Customer's
# modifications. Modification of Sample Code is at Customer's sole and
# exclusive risk. Sample Code may require Customer to have a then
# current license or subscription to the applicable OpenEye offering.
# THE SAMPLE CODE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED.  OPENEYE DISCLAIMS ALL WARRANTIES, INCLUDING, BUT
# NOT LIMITED TO, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE AND NONINFRINGEMENT. In no event shall OpenEye be
# liable for any damages or liability in connection with the Sample Code
# or its use.

from openeye import oechem
from openeye import oedepict
from openeye import oegrapheme

###############################################################
# USED TO GENERATE CODE SNIPPETS FOR THE GRAPHEME DOCUMENTATION
###############################################################


def DepictColorGradient(colorg, opts, basefilename):

    width, height = 400, 100
    image = oedepict.OEImage(width, height)
    oegrapheme.OEDrawColorGradient(image, colorg, opts)

    oedepict.OEWriteImage(basefilename + ".png", image)
    oedepict.OEWriteImage(basefilename + ".pdf", image)

########################################################################
#
########################################################################


width, height = 400, 100
image = oedepict.OEImage(width, height)

# @ <SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-DEFAULT>
colorg = oechem.OELinearColorGradient(oechem.OEColorStop(0.0, oechem.OEYellow))
colorg.AddStop(oechem.OEColorStop(+1.0, oechem.OEOrange))
colorg.AddStop(oechem.OEColorStop(-1.0, oechem.OEGreen))
opts = oegrapheme.OEColorGradientDisplayOptions()
oegrapheme.OEDrawColorGradient(image, colorg)
# @ </SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-DEFAULT>
DepictColorGradient(colorg, opts, "OEColorGradientDisplayOptions_Default")

# @ <SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-COLORSTOPLABELFONT>
opts = oegrapheme.OEColorGradientDisplayOptions()
font = oedepict.OEFont()
font.SetColor(oechem.OEDarkGreen)
font.SetStyle(oedepict.OEFontStyle_Bold)
opts.SetColorStopLabelFont(font)
oegrapheme.OEDrawColorGradient(image, colorg, opts)
# @ </SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-COLORSTOPLABELFONT>
DepictColorGradient(colorg, opts, "OEColorGradientDisplayOptions_SetColorStopLabelFont")

# @ <SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-COLORSTOPLABELFONTSCALE>
opts = oegrapheme.OEColorGradientDisplayOptions()
opts.SetColorStopLabelFontScale(0.5)
oegrapheme.OEDrawColorGradient(image, colorg, opts)
# @ </SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-COLORSTOPLABELFONTSCALE>
DepictColorGradient(colorg, opts, "OEColorGradientDisplayOptions_SetColorStopLabelFontScale")

# @ <SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-COLORSTOPPRECISION>
opts = oegrapheme.OEColorGradientDisplayOptions()
opts.SetColorStopPrecision(4)
oegrapheme.OEDrawColorGradient(image, colorg, opts)
# @ </SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-COLORSTOPPRECISION>
DepictColorGradient(colorg, opts, "OEColorGradientDisplayOptions_SetColorStopPrecision")

# @ <SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-COLORSTOPVISIBILITY>
opts = oegrapheme.OEColorGradientDisplayOptions()
opts.SetColorStopVisibility(False)
oegrapheme.OEDrawColorGradient(image, colorg, opts)
# @ </SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-COLORSTOPVISIBILITY>
DepictColorGradient(colorg, opts, "OEColorGradientDisplayOptions_SetColorStopVisibility")

# @ <SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-BOXRANGE>
opts = oegrapheme.OEColorGradientDisplayOptions()
opts.SetBoxRange(-0.5, 0.5)
oegrapheme.OEDrawColorGradient(image, colorg, opts)
# @ </SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-BOXRANGE>
DepictColorGradient(colorg, opts, "OEColorGradientDisplayOptions_SetBoxRange")

# @ <SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-BOXRANGEPEN>
opts = oegrapheme.OEColorGradientDisplayOptions()
opts.SetBoxRange(-0.5, 0.5)
pen = oedepict.OEPen(oechem.OEBlack, oechem.OEBlack, oedepict.OEFill_On,
                     2.0, oedepict.OEStipple_ShortDash)
opts.SetBoxRangePen(pen)
oegrapheme.OEDrawColorGradient(image, colorg, opts)
# @ </SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-BOXRANGEPEN>
DepictColorGradient(colorg, opts, "OEColorGradientDisplayOptions_SetBoxRangePen")

# @ <SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-ADDMARKEDVALUE>
opts = oegrapheme.OEColorGradientDisplayOptions()
opts.AddMarkedValue(0.20)
oegrapheme.OEDrawColorGradient(image, colorg, opts)
# @ </SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-ADDMARKEDVALUE>
DepictColorGradient(colorg, opts, "OEColorGradientDisplayOptions_AddMarkedValue")

# @ <SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-ADDMARKEDVALUES>
opts = oegrapheme.OEColorGradientDisplayOptions()
markedvalues = oechem.OEDoubleVector([0.20, 0.70, 0.72])
opts.AddMarkedValues(markedvalues)
oegrapheme.OEDrawColorGradient(image, colorg, opts)
# @ </SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-ADDMARKEDVALUES>
DepictColorGradient(colorg, opts, "OEColorGradientDisplayOptions_AddMarkedValues")

# @ <SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-SETMARKEDVALUES>
opts = oegrapheme.OEColorGradientDisplayOptions()
opts.AddMarkedValue(0.0)
markedvalues = oechem.OEDoubleVector([0.5, 0.75])
opts.SetMarkedValues(markedvalues)
oegrapheme.OEDrawColorGradient(image, colorg, opts)
# @ </SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-SETMARKEDVALUES>
DepictColorGradient(colorg, opts, "OEColorGradientDisplayOptions_SetMarkedValues")

# @ <SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-MARKEDVALUEPRECISION>
opts = oegrapheme.OEColorGradientDisplayOptions()
markedvalues = oechem.OEDoubleVector([0.20, 0.70, 0.72])
opts.AddMarkedValues(markedvalues)
opts.SetMarkedValuePrecision(1)
oegrapheme.OEDrawColorGradient(image, colorg, opts)
# @ </SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-MARKEDVALUEPRECISION>
DepictColorGradient(colorg, opts, "OEColorGradientDisplayOptions_SetMarkedValuePrecision")

# @ <SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-MARKEDVALUEPEN>
opts = oegrapheme.OEColorGradientDisplayOptions()
markedvalues = oechem.OEDoubleVector([0.20, 0.70, 0.72])
opts.AddMarkedValues(markedvalues)
pen = oedepict.OEPen(oechem.OEBlack, oechem.OEDarkGreen, oedepict.OEFill_On,
                     2.0, oedepict.OEStipple_ShortDash)
opts.SetMarkedValuePen(pen)
oegrapheme.OEDrawColorGradient(image, colorg, opts)
# @ </SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-MARKEDVALUEPEN>
DepictColorGradient(colorg, opts, "OEColorGradientDisplayOptions_SetMarkedValuePen")

# @ <SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-ADDLABEL>
opts = oegrapheme.OEColorGradientDisplayOptions()
opts.AddLabel(oegrapheme.OEColorGradientLabel(-1.0, "dissimilar"))
opts.AddLabel(oegrapheme.OEColorGradientLabel(+1.0, "similar"))
oegrapheme.OEDrawColorGradient(image, colorg, opts)
# @ </SNIPPET-OECOLORGRADIENTDISPLAYOPTIONS-ADDLABEL>
DepictColorGradient(colorg, opts, "OEColorGradientDisplayOptions_AddLabel")
